DEF = 'urn:mace:dir:attribute-def:'


MAP = {
    "identifier": "urn:oasis:names:tc:SAML:2.0:attrname-format:basic",
    'fro': {
        DEF+'aRecord': 'aRecord',
        DEF+'aliasedEntryName': 'aliasedEntryName',
        DEF+'aliasedObjectName': 'aliasedObjectName',
        DEF+'associatedDomain': 'associatedDomain',
        DEF+'associatedName': 'associatedName',
        DEF+'audio': 'audio',
        DEF+'authorityRevocationList': 'authorityRevocationList',
        DEF+'buildingName': 'buildingName',
        DEF+'businessCategory': 'businessCategory',
        DEF+'c': 'c',
        DEF+'cACertificate': 'cACertificate',
        DEF+'cNAMERecord': 'cNAMERecord',
        DEF+'carLicense': 'carLicense',
        DEF+'certificateRevocationList': 'certificateRevocationList',
        DEF+'cn': 'cn',
        DEF+'co': 'co',
        DEF+'commonName': 'commonName',
        DEF+'countryName': 'countryName',
        DEF+'crossCertificatePair': 'crossCertificatePair',
        DEF+'dITRedirect': 'dITRedirect',
        DEF+'dSAQuality': 'dSAQuality',
        DEF+'dc': 'dc',
        DEF+'deltaRevocationList': 'deltaRevocationList',
        DEF+'departmentNumber': 'departmentNumber',
        DEF+'description': 'description',
        DEF+'destinationIndicator': 'destinationIndicator',
        DEF+'displayName': 'displayName',
        DEF+'distinguishedName': 'distinguishedName',
        DEF+'dmdName': 'dmdName',
        DEF+'dnQualifier': 'dnQualifier',
        DEF+'documentAuthor': 'documentAuthor',
        DEF+'documentIdentifier': 'documentIdentifier',
        DEF+'documentLocation': 'documentLocation',
        DEF+'documentPublisher': 'documentPublisher',
        DEF+'documentTitle': 'documentTitle',
        DEF+'documentVersion': 'documentVersion',
        DEF+'domainComponent': 'domainComponent',
        DEF+'drink': 'drink',
        DEF+'eduOrgHomePageURI': 'eduOrgHomePageURI',
        DEF+'eduOrgIdentityAuthNPolicyURI': 'eduOrgIdentityAuthNPolicyURI',
        DEF+'eduOrgLegalName': 'eduOrgLegalName',
        DEF+'eduOrgSuperiorURI': 'eduOrgSuperiorURI',
        DEF+'eduOrgWhitePagesURI': 'eduOrgWhitePagesURI',
        DEF+'eduCourseOffering': 'eduCourseOffering',
        DEF+'eduCourseMember': 'eduCourseMember',
        DEF+'eduPersonAffiliation': 'eduPersonAffiliation',
        DEF+'eduPersonEntitlement': 'eduPersonEntitlement',
        DEF+'eduPersonNickname': 'eduPersonNickname',
        DEF+'eduPersonOrgDN': 'eduPersonOrgDN',
        DEF+'eduPersonOrgUnitDN': 'eduPersonOrgUnitDN',
        DEF+'eduPersonPrimaryAffiliation': 'eduPersonPrimaryAffiliation',
        DEF+'eduPersonPrimaryOrgUnitDN': 'eduPersonPrimaryOrgUnitDN',
        DEF+'eduPersonPrincipalName': 'eduPersonPrincipalName',
        DEF+'eduPersonPrincipalNamePrior': 'eduPersonPrincipalNamePrior',
        DEF+'eduPersonScopedAffiliation': 'eduPersonScopedAffiliation',
        DEF+'eduPersonTargetedID': 'eduPersonTargetedID',
        DEF+'eduPersonAssurance': 'eduPersonAssurance',
        DEF+'eduPersonUniqueId': 'eduPersonUniqueId',
        DEF+'eduPersonOrcid': 'eduPersonOrcid',
        DEF+'email': 'email',
        DEF+'emailAddress': 'emailAddress',
        DEF+'employeeNumber': 'employeeNumber',
        DEF+'employeeType': 'employeeType',
        DEF+'enhancedSearchGuide': 'enhancedSearchGuide',
        DEF+'facsimileTelephoneNumber': 'facsimileTelephoneNumber',
        DEF+'favouriteDrink': 'favouriteDrink',
        DEF+'fax': 'fax',
        DEF+'federationFeideSchemaVersion': 'federationFeideSchemaVersion',
        DEF+'friendlyCountryName': 'friendlyCountryName',
        DEF+'generationQualifier': 'generationQualifier',
        DEF+'givenName': 'givenName',
        DEF+'gn': 'gn',
        DEF+'homePhone': 'homePhone',
        DEF+'homePostalAddress': 'homePostalAddress',
        DEF+'homeTelephoneNumber': 'homeTelephoneNumber',
        DEF+'host': 'host',
        DEF+'houseIdentifier': 'houseIdentifier',
        DEF+'info': 'info',
        DEF+'initials': 'initials',
        DEF+'internationaliSDNNumber': 'internationaliSDNNumber',
        DEF+'janetMailbox': 'janetMailbox',
        DEF+'jpegPhoto': 'jpegPhoto',
        DEF+'knowledgeInformation': 'knowledgeInformation',
        DEF+'l': 'l',
        DEF+'labeledURI': 'labeledURI',
        DEF+'localityName': 'localityName',
        DEF+'mDRecord': 'mDRecord',
        DEF+'mXRecord': 'mXRecord',
        DEF+'mail': 'mail',
        DEF+'mailPreferenceOption': 'mailPreferenceOption',
        DEF+'manager': 'manager',
        DEF+'member': 'member',
        DEF+'mobile': 'mobile',
        DEF+'mobileTelephoneNumber': 'mobileTelephoneNumber',
        DEF+'nSRecord': 'nSRecord',
        DEF+'name': 'name',
        DEF+'norEduOrgAcronym': 'norEduOrgAcronym',
        DEF+'norEduOrgNIN': 'norEduOrgNIN',
        DEF+'norEduOrgSchemaVersion': 'norEduOrgSchemaVersion',
        DEF+'norEduOrgUniqueIdentifier': 'norEduOrgUniqueIdentifier',
        DEF+'norEduOrgUniqueNumber': 'norEduOrgUniqueNumber',
        DEF+'norEduOrgUnitUniqueIdentifier': 'norEduOrgUnitUniqueIdentifier',
        DEF+'norEduOrgUnitUniqueNumber': 'norEduOrgUnitUniqueNumber',
        DEF+'norEduPersonBirthDate': 'norEduPersonBirthDate',
        DEF+'norEduPersonLIN': 'norEduPersonLIN',
        DEF+'norEduPersonNIN': 'norEduPersonNIN',
        DEF+'o': 'o',
        DEF+'objectClass': 'objectClass',
        DEF+'organizationName': 'organizationName',
        DEF+'organizationalStatus': 'organizationalStatus',
        DEF+'organizationalUnitName': 'organizationalUnitName',
        DEF+'otherMailbox': 'otherMailbox',
        DEF+'ou': 'ou',
        DEF+'owner': 'owner',
        DEF+'pager': 'pager',
        DEF+'pagerTelephoneNumber': 'pagerTelephoneNumber',
        DEF+'personalSignature': 'personalSignature',
        DEF+'personalTitle': 'personalTitle',
        DEF+'photo': 'photo',
        DEF+'physicalDeliveryOfficeName': 'physicalDeliveryOfficeName',
        DEF+'pkcs9email': 'pkcs9email',
        DEF+'postOfficeBox': 'postOfficeBox',
        DEF+'postalAddress': 'postalAddress',
        DEF+'postalCode': 'postalCode',
        DEF+'preferredDeliveryMethod': 'preferredDeliveryMethod',
        DEF+'preferredLanguage': 'preferredLanguage',
        DEF+'presentationAddress': 'presentationAddress',
        DEF+'protocolInformation': 'protocolInformation',
        DEF+'pseudonym': 'pseudonym',
        DEF+'registeredAddress': 'registeredAddress',
        DEF+'rfc822Mailbox': 'rfc822Mailbox',
        DEF+'roleOccupant': 'roleOccupant',
        DEF+'roomNumber': 'roomNumber',
        DEF+'sOARecord': 'sOARecord',
        DEF+'searchGuide': 'searchGuide',
        DEF+'secretary': 'secretary',
        DEF+'seeAlso': 'seeAlso',
        DEF+'serialNumber': 'serialNumber',
        DEF+'singleLevelQuality': 'singleLevelQuality',
        DEF+'sn': 'sn',
        DEF+'st': 'st',
        DEF+'stateOrProvinceName': 'stateOrProvinceName',
        DEF+'street': 'street',
        DEF+'streetAddress': 'streetAddress',
        DEF+'subtreeMaximumQuality': 'subtreeMaximumQuality',
        DEF+'subtreeMinimumQuality': 'subtreeMinimumQuality',
        DEF+'supportedAlgorithms': 'supportedAlgorithms',
        DEF+'supportedApplicationContext': 'supportedApplicationContext',
        DEF+'surname': 'surname',
        DEF+'telephoneNumber': 'telephoneNumber',
        DEF+'teletexTerminalIdentifier': 'teletexTerminalIdentifier',
        DEF+'telexNumber': 'telexNumber',
        DEF+'textEncodedORAddress': 'textEncodedORAddress',
        DEF+'title': 'title',
        DEF+'uid': 'uid',
        DEF+'uniqueIdentifier': 'uniqueIdentifier',
        DEF+'uniqueMember': 'uniqueMember',
        DEF+'userCertificate': 'userCertificate',
        DEF+'userClass': 'userClass',
        DEF+'userPKCS12': 'userPKCS12',
        DEF+'userPassword': 'userPassword',
        DEF+'userSMIMECertificate': 'userSMIMECertificate',
        DEF+'userid': 'userid',
        DEF+'x121Address': 'x121Address',
        DEF+'x500UniqueIdentifier': 'x500UniqueIdentifier',
    },
    'to': {
        'aRecord': DEF+'aRecord',
        'aliasedEntryName': DEF+'aliasedEntryName',
        'aliasedObjectName': DEF+'aliasedObjectName',
        'associatedDomain': DEF+'associatedDomain',
        'associatedName': DEF+'associatedName',
        'audio': DEF+'audio',
        'authorityRevocationList': DEF+'authorityRevocationList',
        'buildingName': DEF+'buildingName',
        'businessCategory': DEF+'businessCategory',
        'c': DEF+'c',
        'cACertificate': DEF+'cACertificate',
        'cNAMERecord': DEF+'cNAMERecord',
        'carLicense': DEF+'carLicense',
        'certificateRevocationList': DEF+'certificateRevocationList',
        'cn': DEF+'cn',
        'co': DEF+'co',
        'commonName': DEF+'commonName',
        'countryName': DEF+'countryName',
        'crossCertificatePair': DEF+'crossCertificatePair',
        'dITRedirect': DEF+'dITRedirect',
        'dSAQuality': DEF+'dSAQuality',
        'dc': DEF+'dc',
        'deltaRevocationList': DEF+'deltaRevocationList',
        'departmentNumber': DEF+'departmentNumber',
        'description': DEF+'description',
        'destinationIndicator': DEF+'destinationIndicator',
        'displayName': DEF+'displayName',
        'distinguishedName': DEF+'distinguishedName',
        'dmdName': DEF+'dmdName',
        'dnQualifier': DEF+'dnQualifier',
        'documentAuthor': DEF+'documentAuthor',
        'documentIdentifier': DEF+'documentIdentifier',
        'documentLocation': DEF+'documentLocation',
        'documentPublisher': DEF+'documentPublisher',
        'documentTitle': DEF+'documentTitle',
        'documentVersion': DEF+'documentVersion',
        'domainComponent': DEF+'domainComponent',
        'drink': DEF+'drink',
        'eduOrgHomePageURI': DEF+'eduOrgHomePageURI',
        'eduOrgIdentityAuthNPolicyURI': DEF+'eduOrgIdentityAuthNPolicyURI',
        'eduOrgLegalName': DEF+'eduOrgLegalName',
        'eduOrgSuperiorURI': DEF+'eduOrgSuperiorURI',
        'eduOrgWhitePagesURI': DEF+'eduOrgWhitePagesURI',
        'eduCourseMember': DEF+'eduCourseMember',
        'eduCourseOffering': DEF+'eduCourseOffering',
        'eduPersonAffiliation': DEF+'eduPersonAffiliation',
        'eduPersonEntitlement': DEF+'eduPersonEntitlement',
        'eduPersonNickname': DEF+'eduPersonNickname',
        'eduPersonOrgDN': DEF+'eduPersonOrgDN',
        'eduPersonOrgUnitDN': DEF+'eduPersonOrgUnitDN',
        'eduPersonPrimaryAffiliation': DEF+'eduPersonPrimaryAffiliation',
        'eduPersonPrimaryOrgUnitDN': DEF+'eduPersonPrimaryOrgUnitDN',
        'eduPersonPrincipalName': DEF+'eduPersonPrincipalName',
        'eduPersonPrincipalNamePrior': DEF+'eduPersonPrincipalNamePrior',
        'eduPersonScopedAffiliation': DEF+'eduPersonScopedAffiliation',
        'eduPersonTargetedID': DEF+'eduPersonTargetedID',
        'eduPersonAssurance': DEF+'eduPersonAssurance',
        'eduPersonUniqueId': DEF+'eduPersonUniqueId',
        'eduPersonOrcid': DEF+'eduPersonOrcid',
        'email': DEF+'email',
        'emailAddress': DEF+'emailAddress',
        'employeeNumber': DEF+'employeeNumber',
        'employeeType': DEF+'employeeType',
        'enhancedSearchGuide': DEF+'enhancedSearchGuide',
        'facsimileTelephoneNumber': DEF+'facsimileTelephoneNumber',
        'favouriteDrink': DEF+'favouriteDrink',
        'fax': DEF+'fax',
        'federationFeideSchemaVersion': DEF+'federationFeideSchemaVersion',
        'friendlyCountryName': DEF+'friendlyCountryName',
        'generationQualifier': DEF+'generationQualifier',
        'givenName': DEF+'givenName',
        'gn': DEF+'gn',
        'homePhone': DEF+'homePhone',
        'homePostalAddress': DEF+'homePostalAddress',
        'homeTelephoneNumber': DEF+'homeTelephoneNumber',
        'host': DEF+'host',
        'houseIdentifier': DEF+'houseIdentifier',
        'info': DEF+'info',
        'initials': DEF+'initials',
        'internationaliSDNNumber': DEF+'internationaliSDNNumber',
        'janetMailbox': DEF+'janetMailbox',
        'jpegPhoto': DEF+'jpegPhoto',
        'knowledgeInformation': DEF+'knowledgeInformation',
        'l': DEF+'l',
        'labeledURI': DEF+'labeledURI',
        'localityName': DEF+'localityName',
        'mDRecord': DEF+'mDRecord',
        'mXRecord': DEF+'mXRecord',
        'mail': DEF+'mail',
        'mailPreferenceOption': DEF+'mailPreferenceOption',
        'manager': DEF+'manager',
        'member': DEF+'member',
        'mobile': DEF+'mobile',
        'mobileTelephoneNumber': DEF+'mobileTelephoneNumber',
        'nSRecord': DEF+'nSRecord',
        'name': DEF+'name',
        'norEduOrgAcronym': DEF+'norEduOrgAcronym',
        'norEduOrgNIN': DEF+'norEduOrgNIN',
        'norEduOrgSchemaVersion': DEF+'norEduOrgSchemaVersion',
        'norEduOrgUniqueIdentifier': DEF+'norEduOrgUniqueIdentifier',
        'norEduOrgUniqueNumber': DEF+'norEduOrgUniqueNumber',
        'norEduOrgUnitUniqueIdentifier': DEF+'norEduOrgUnitUniqueIdentifier',
        'norEduOrgUnitUniqueNumber': DEF+'norEduOrgUnitUniqueNumber',
        'norEduPersonBirthDate': DEF+'norEduPersonBirthDate',
        'norEduPersonLIN': DEF+'norEduPersonLIN',
        'norEduPersonNIN': DEF+'norEduPersonNIN',
        'o': DEF+'o',
        'objectClass': DEF+'objectClass',
        'organizationName': DEF+'organizationName',
        'organizationalStatus': DEF+'organizationalStatus',
        'organizationalUnitName': DEF+'organizationalUnitName',
        'otherMailbox': DEF+'otherMailbox',
        'ou': DEF+'ou',
        'owner': DEF+'owner',
        'pager': DEF+'pager',
        'pagerTelephoneNumber': DEF+'pagerTelephoneNumber',
        'personalSignature': DEF+'personalSignature',
        'personalTitle': DEF+'personalTitle',
        'photo': DEF+'photo',
        'physicalDeliveryOfficeName': DEF+'physicalDeliveryOfficeName',
        'pkcs9email': DEF+'pkcs9email',
        'postOfficeBox': DEF+'postOfficeBox',
        'postalAddress': DEF+'postalAddress',
        'postalCode': DEF+'postalCode',
        'preferredDeliveryMethod': DEF+'preferredDeliveryMethod',
        'preferredLanguage': DEF+'preferredLanguage',
        'presentationAddress': DEF+'presentationAddress',
        'protocolInformation': DEF+'protocolInformation',
        'pseudonym': DEF+'pseudonym',
        'registeredAddress': DEF+'registeredAddress',
        'rfc822Mailbox': DEF+'rfc822Mailbox',
        'roleOccupant': DEF+'roleOccupant',
        'roomNumber': DEF+'roomNumber',
        'sOARecord': DEF+'sOARecord',
        'searchGuide': DEF+'searchGuide',
        'secretary': DEF+'secretary',
        'seeAlso': DEF+'seeAlso',
        'serialNumber': DEF+'serialNumber',
        'singleLevelQuality': DEF+'singleLevelQuality',
        'sn': DEF+'sn',
        'st': DEF+'st',
        'stateOrProvinceName': DEF+'stateOrProvinceName',
        'street': DEF+'street',
        'streetAddress': DEF+'streetAddress',
        'subtreeMaximumQuality': DEF+'subtreeMaximumQuality',
        'subtreeMinimumQuality': DEF+'subtreeMinimumQuality',
        'supportedAlgorithms': DEF+'supportedAlgorithms',
        'supportedApplicationContext': DEF+'supportedApplicationContext',
        'surname': DEF+'surname',
        'telephoneNumber': DEF+'telephoneNumber',
        'teletexTerminalIdentifier': DEF+'teletexTerminalIdentifier',
        'telexNumber': DEF+'telexNumber',
        'textEncodedORAddress': DEF+'textEncodedORAddress',
        'title': DEF+'title',
        'uid': DEF+'uid',
        'uniqueIdentifier': DEF+'uniqueIdentifier',
        'uniqueMember': DEF+'uniqueMember',
        'userCertificate': DEF+'userCertificate',
        'userClass': DEF+'userClass',
        'userPKCS12': DEF+'userPKCS12',
        'userPassword': DEF+'userPassword',
        'userSMIMECertificate': DEF+'userSMIMECertificate',
        'userid': DEF+'userid',
        'x121Address': DEF+'x121Address',
        'x500UniqueIdentifier': DEF+'x500UniqueIdentifier',
    }
}
