
MAP = {
    "identifier": "urn:oasis:names:tc:SAML:2.0:attrname-format:basic",
    "fro": {
        'urn:mace:dir:attribute-def:aRecord': 'aRecord',
        'urn:mace:dir:attribute-def:aliasedEntryName': 'aliasedEntryName',
        'urn:mace:dir:attribute-def:aliasedObjectName': 'aliasedObjectName',
        'urn:mace:dir:attribute-def:associatedDomain': 'associatedDomain',
        'urn:mace:dir:attribute-def:associatedName': 'associatedName',
        'urn:mace:dir:attribute-def:audio': 'audio',
        'urn:mace:dir:attribute-def:authorityRevocationList': 'authorityRevocationList',
        'urn:mace:dir:attribute-def:buildingName': 'buildingName',
        'urn:mace:dir:attribute-def:businessCategory': 'businessCategory',
        'urn:mace:dir:attribute-def:c': 'c',
        'urn:mace:dir:attribute-def:cACertificate': 'cACertificate',
        'urn:mace:dir:attribute-def:cNAMERecord': 'cNAMERecord',
        'urn:mace:dir:attribute-def:carLicense': 'carLicense',
        'urn:mace:dir:attribute-def:certificateRevocationList': 'certificateRevocationList',
        'urn:mace:dir:attribute-def:cn': 'cn',
        'urn:mace:dir:attribute-def:co': 'co',
        'urn:mace:dir:attribute-def:commonName': 'commonName',
        'urn:mace:dir:attribute-def:countryName': 'countryName',
        'urn:mace:dir:attribute-def:crossCertificatePair': 'crossCertificatePair',
        'urn:mace:dir:attribute-def:dITRedirect': 'dITRedirect',
        'urn:mace:dir:attribute-def:dSAQuality': 'dSAQuality',
        'urn:mace:dir:attribute-def:dc': 'dc',
        'urn:mace:dir:attribute-def:deltaRevocationList': 'deltaRevocationList',
        'urn:mace:dir:attribute-def:departmentNumber': 'departmentNumber',
        'urn:mace:dir:attribute-def:description': 'description',
        'urn:mace:dir:attribute-def:destinationIndicator': 'destinationIndicator',
        'urn:mace:dir:attribute-def:displayName': 'displayName',
        'urn:mace:dir:attribute-def:distinguishedName': 'distinguishedName',
        'urn:mace:dir:attribute-def:dmdName': 'dmdName',
        'urn:mace:dir:attribute-def:dnQualifier': 'dnQualifier',
        'urn:mace:dir:attribute-def:documentAuthor': 'documentAuthor',
        'urn:mace:dir:attribute-def:documentIdentifier': 'documentIdentifier',
        'urn:mace:dir:attribute-def:documentLocation': 'documentLocation',
        'urn:mace:dir:attribute-def:documentPublisher': 'documentPublisher',
        'urn:mace:dir:attribute-def:documentTitle': 'documentTitle',
        'urn:mace:dir:attribute-def:documentVersion': 'documentVersion',
        'urn:mace:dir:attribute-def:domainComponent': 'domainComponent',
        'urn:mace:dir:attribute-def:drink': 'drink',
        'urn:mace:dir:attribute-def:eduOrgHomePageURI': 'eduOrgHomePageURI',
        'urn:mace:dir:attribute-def:eduOrgIdentityAuthNPolicyURI': 'eduOrgIdentityAuthNPolicyURI',
        'urn:mace:dir:attribute-def:eduOrgLegalName': 'eduOrgLegalName',
        'urn:mace:dir:attribute-def:eduOrgSuperiorURI': 'eduOrgSuperiorURI',
        'urn:mace:dir:attribute-def:eduOrgWhitePagesURI': 'eduOrgWhitePagesURI',
        'urn:mace:dir:attribute-def:eduPersonAffiliation': 'eduPersonAffiliation',
        'urn:mace:dir:attribute-def:eduPersonEntitlement': 'eduPersonEntitlement',
        'urn:mace:dir:attribute-def:eduPersonNickname': 'eduPersonNickname',
        'urn:mace:dir:attribute-def:eduPersonOrgDN': 'eduPersonOrgDN',
        'urn:mace:dir:attribute-def:eduPersonOrgUnitDN': 'eduPersonOrgUnitDN',
        'urn:mace:dir:attribute-def:eduPersonPrimaryAffiliation': 'eduPersonPrimaryAffiliation',
        'urn:mace:dir:attribute-def:eduPersonPrimaryOrgUnitDN': 'eduPersonPrimaryOrgUnitDN',
        'urn:mace:dir:attribute-def:eduPersonPrincipalName': 'eduPersonPrincipalName',
        'urn:mace:dir:attribute-def:eduPersonScopedAffiliation': 'eduPersonScopedAffiliation',
        'urn:mace:dir:attribute-def:eduPersonTargetedID': 'eduPersonTargetedID',
        'urn:mace:dir:attribute-def:email': 'email',
        'urn:mace:dir:attribute-def:emailAddress': 'emailAddress',
        'urn:mace:dir:attribute-def:employeeNumber': 'employeeNumber',
        'urn:mace:dir:attribute-def:employeeType': 'employeeType',
        'urn:mace:dir:attribute-def:enhancedSearchGuide': 'enhancedSearchGuide',
        'urn:mace:dir:attribute-def:facsimileTelephoneNumber': 'facsimileTelephoneNumber',
        'urn:mace:dir:attribute-def:favouriteDrink': 'favouriteDrink',
        'urn:mace:dir:attribute-def:fax': 'fax',
        'urn:mace:dir:attribute-def:federationFeideSchemaVersion': 'federationFeideSchemaVersion',
        'urn:mace:dir:attribute-def:friendlyCountryName': 'friendlyCountryName',
        'urn:mace:dir:attribute-def:generationQualifier': 'generationQualifier',
        'urn:mace:dir:attribute-def:givenName': 'givenName',
        'urn:mace:dir:attribute-def:gn': 'gn',
        'urn:mace:dir:attribute-def:homePhone': 'homePhone',
        'urn:mace:dir:attribute-def:homePostalAddress': 'homePostalAddress',
        'urn:mace:dir:attribute-def:homeTelephoneNumber': 'homeTelephoneNumber',
        'urn:mace:dir:attribute-def:host': 'host',
        'urn:mace:dir:attribute-def:houseIdentifier': 'houseIdentifier',
        'urn:mace:dir:attribute-def:info': 'info',
        'urn:mace:dir:attribute-def:initials': 'initials',
        'urn:mace:dir:attribute-def:internationaliSDNNumber': 'internationaliSDNNumber',
        'urn:mace:dir:attribute-def:janetMailbox': 'janetMailbox',
        'urn:mace:dir:attribute-def:jpegPhoto': 'jpegPhoto',
        'urn:mace:dir:attribute-def:knowledgeInformation': 'knowledgeInformation',
        'urn:mace:dir:attribute-def:l': 'l',
        'urn:mace:dir:attribute-def:labeledURI': 'labeledURI',
        'urn:mace:dir:attribute-def:localityName': 'localityName',
        'urn:mace:dir:attribute-def:mDRecord': 'mDRecord',
        'urn:mace:dir:attribute-def:mXRecord': 'mXRecord',
        'urn:mace:dir:attribute-def:mail': 'mail',
        'urn:mace:dir:attribute-def:mailPreferenceOption': 'mailPreferenceOption',
        'urn:mace:dir:attribute-def:manager': 'manager',
        'urn:mace:dir:attribute-def:member': 'member',
        'urn:mace:dir:attribute-def:mobile': 'mobile',
        'urn:mace:dir:attribute-def:mobileTelephoneNumber': 'mobileTelephoneNumber',
        'urn:mace:dir:attribute-def:nSRecord': 'nSRecord',
        'urn:mace:dir:attribute-def:name': 'name',
        'urn:mace:dir:attribute-def:norEduOrgAcronym': 'norEduOrgAcronym',
        'urn:mace:dir:attribute-def:norEduOrgNIN': 'norEduOrgNIN',
        'urn:mace:dir:attribute-def:norEduOrgSchemaVersion': 'norEduOrgSchemaVersion',
        'urn:mace:dir:attribute-def:norEduOrgUniqueIdentifier': 'norEduOrgUniqueIdentifier',
        'urn:mace:dir:attribute-def:norEduOrgUniqueNumber': 'norEduOrgUniqueNumber',
        'urn:mace:dir:attribute-def:norEduOrgUnitUniqueIdentifier': 'norEduOrgUnitUniqueIdentifier',
        'urn:mace:dir:attribute-def:norEduOrgUnitUniqueNumber': 'norEduOrgUnitUniqueNumber',
        'urn:mace:dir:attribute-def:norEduPersonBirthDate': 'norEduPersonBirthDate',
        'urn:mace:dir:attribute-def:norEduPersonLIN': 'norEduPersonLIN',
        'urn:mace:dir:attribute-def:norEduPersonNIN': 'norEduPersonNIN',
        'urn:mace:dir:attribute-def:o': 'o',
        'urn:mace:dir:attribute-def:objectClass': 'objectClass',
        'urn:mace:dir:attribute-def:organizationName': 'organizationName',
        'urn:mace:dir:attribute-def:organizationalStatus': 'organizationalStatus',
        'urn:mace:dir:attribute-def:organizationalUnitName': 'organizationalUnitName',
        'urn:mace:dir:attribute-def:otherMailbox': 'otherMailbox',
        'urn:mace:dir:attribute-def:ou': 'ou',
        'urn:mace:dir:attribute-def:owner': 'owner',
        'urn:mace:dir:attribute-def:pager': 'pager',
        'urn:mace:dir:attribute-def:pagerTelephoneNumber': 'pagerTelephoneNumber',
        'urn:mace:dir:attribute-def:personalSignature': 'personalSignature',
        'urn:mace:dir:attribute-def:personalTitle': 'personalTitle',
        'urn:mace:dir:attribute-def:photo': 'photo',
        'urn:mace:dir:attribute-def:physicalDeliveryOfficeName': 'physicalDeliveryOfficeName',
        'urn:mace:dir:attribute-def:pkcs9email': 'pkcs9email',
        'urn:mace:dir:attribute-def:postOfficeBox': 'postOfficeBox',
        'urn:mace:dir:attribute-def:postalAddress': 'postalAddress',
        'urn:mace:dir:attribute-def:postalCode': 'postalCode',
        'urn:mace:dir:attribute-def:preferredDeliveryMethod': 'preferredDeliveryMethod',
        'urn:mace:dir:attribute-def:preferredLanguage': 'preferredLanguage',
        'urn:mace:dir:attribute-def:presentationAddress': 'presentationAddress',
        'urn:mace:dir:attribute-def:protocolInformation': 'protocolInformation',
        'urn:mace:dir:attribute-def:pseudonym': 'pseudonym',
        'urn:mace:dir:attribute-def:registeredAddress': 'registeredAddress',
        'urn:mace:dir:attribute-def:rfc822Mailbox': 'rfc822Mailbox',
        'urn:mace:dir:attribute-def:roleOccupant': 'roleOccupant',
        'urn:mace:dir:attribute-def:roomNumber': 'roomNumber',
        'urn:mace:dir:attribute-def:sOARecord': 'sOARecord',
        'urn:mace:dir:attribute-def:searchGuide': 'searchGuide',
        'urn:mace:dir:attribute-def:secretary': 'secretary',
        'urn:mace:dir:attribute-def:seeAlso': 'seeAlso',
        'urn:mace:dir:attribute-def:serialNumber': 'serialNumber',
        'urn:mace:dir:attribute-def:singleLevelQuality': 'singleLevelQuality',
        'urn:mace:dir:attribute-def:sn': 'sn',
        'urn:mace:dir:attribute-def:st': 'st',
        'urn:mace:dir:attribute-def:stateOrProvinceName': 'stateOrProvinceName',
        'urn:mace:dir:attribute-def:street': 'street',
        'urn:mace:dir:attribute-def:streetAddress': 'streetAddress',
        'urn:mace:dir:attribute-def:subtreeMaximumQuality': 'subtreeMaximumQuality',
        'urn:mace:dir:attribute-def:subtreeMinimumQuality': 'subtreeMinimumQuality',
        'urn:mace:dir:attribute-def:supportedAlgorithms': 'supportedAlgorithms',
        'urn:mace:dir:attribute-def:supportedApplicationContext': 'supportedApplicationContext',
        'urn:mace:dir:attribute-def:surname': 'surname',
        'urn:mace:dir:attribute-def:telephoneNumber': 'telephoneNumber',
        'urn:mace:dir:attribute-def:teletexTerminalIdentifier': 'teletexTerminalIdentifier',
        'urn:mace:dir:attribute-def:telexNumber': 'telexNumber',
        'urn:mace:dir:attribute-def:textEncodedORAddress': 'textEncodedORAddress',
        'urn:mace:dir:attribute-def:title': 'title',
        'urn:mace:dir:attribute-def:uid': 'uid',
        'urn:mace:dir:attribute-def:uniqueIdentifier': 'uniqueIdentifier',
        'urn:mace:dir:attribute-def:uniqueMember': 'uniqueMember',
        'urn:mace:dir:attribute-def:userCertificate': 'userCertificate',
        'urn:mace:dir:attribute-def:userClass': 'userClass',
        'urn:mace:dir:attribute-def:userPKCS12': 'userPKCS12',
        'urn:mace:dir:attribute-def:userPassword': 'userPassword',
        'urn:mace:dir:attribute-def:userSMIMECertificate': 'userSMIMECertificate',
        'urn:mace:dir:attribute-def:userid': 'userid',
        'urn:mace:dir:attribute-def:x121Address': 'x121Address',
        'urn:mace:dir:attribute-def:x500UniqueIdentifier': 'x500UniqueIdentifier',
        },
    "to": {
        'aRecord': 'urn:mace:dir:attribute-def:aRecord',
        'aliasedEntryName': 'urn:mace:dir:attribute-def:aliasedEntryName',
        'aliasedObjectName': 'urn:mace:dir:attribute-def:aliasedObjectName',
        'associatedDomain': 'urn:mace:dir:attribute-def:associatedDomain',
        'associatedName': 'urn:mace:dir:attribute-def:associatedName',
        'audio': 'urn:mace:dir:attribute-def:audio',
        'authorityRevocationList': 'urn:mace:dir:attribute-def:authorityRevocationList',
        'buildingName': 'urn:mace:dir:attribute-def:buildingName',
        'businessCategory': 'urn:mace:dir:attribute-def:businessCategory',
        'c': 'urn:mace:dir:attribute-def:c',
        'cACertificate': 'urn:mace:dir:attribute-def:cACertificate',
        'cNAMERecord': 'urn:mace:dir:attribute-def:cNAMERecord',
        'carLicense': 'urn:mace:dir:attribute-def:carLicense',
        'certificateRevocationList': 'urn:mace:dir:attribute-def:certificateRevocationList',
        'cn': 'urn:mace:dir:attribute-def:cn',
        'co': 'urn:mace:dir:attribute-def:co',
        'commonName': 'urn:mace:dir:attribute-def:commonName',
        'countryName': 'urn:mace:dir:attribute-def:countryName',
        'crossCertificatePair': 'urn:mace:dir:attribute-def:crossCertificatePair',
        'dITRedirect': 'urn:mace:dir:attribute-def:dITRedirect',
        'dSAQuality': 'urn:mace:dir:attribute-def:dSAQuality',
        'dc': 'urn:mace:dir:attribute-def:dc',
        'deltaRevocationList': 'urn:mace:dir:attribute-def:deltaRevocationList',
        'departmentNumber': 'urn:mace:dir:attribute-def:departmentNumber',
        'description': 'urn:mace:dir:attribute-def:description',
        'destinationIndicator': 'urn:mace:dir:attribute-def:destinationIndicator',
        'displayName': 'urn:mace:dir:attribute-def:displayName',
        'distinguishedName': 'urn:mace:dir:attribute-def:distinguishedName',
        'dmdName': 'urn:mace:dir:attribute-def:dmdName',
        'dnQualifier': 'urn:mace:dir:attribute-def:dnQualifier',
        'documentAuthor': 'urn:mace:dir:attribute-def:documentAuthor',
        'documentIdentifier': 'urn:mace:dir:attribute-def:documentIdentifier',
        'documentLocation': 'urn:mace:dir:attribute-def:documentLocation',
        'documentPublisher': 'urn:mace:dir:attribute-def:documentPublisher',
        'documentTitle': 'urn:mace:dir:attribute-def:documentTitle',
        'documentVersion': 'urn:mace:dir:attribute-def:documentVersion',
        'domainComponent': 'urn:mace:dir:attribute-def:domainComponent',
        'drink': 'urn:mace:dir:attribute-def:drink',
        'eduOrgHomePageURI': 'urn:mace:dir:attribute-def:eduOrgHomePageURI',
        'eduOrgIdentityAuthNPolicyURI': 'urn:mace:dir:attribute-def:eduOrgIdentityAuthNPolicyURI',
        'eduOrgLegalName': 'urn:mace:dir:attribute-def:eduOrgLegalName',
        'eduOrgSuperiorURI': 'urn:mace:dir:attribute-def:eduOrgSuperiorURI',
        'eduOrgWhitePagesURI': 'urn:mace:dir:attribute-def:eduOrgWhitePagesURI',
        'eduPersonAffiliation': 'urn:mace:dir:attribute-def:eduPersonAffiliation',
        'eduPersonEntitlement': 'urn:mace:dir:attribute-def:eduPersonEntitlement',
        'eduPersonNickname': 'urn:mace:dir:attribute-def:eduPersonNickname',
        'eduPersonOrgDN': 'urn:mace:dir:attribute-def:eduPersonOrgDN',
        'eduPersonOrgUnitDN': 'urn:mace:dir:attribute-def:eduPersonOrgUnitDN',
        'eduPersonPrimaryAffiliation': 'urn:mace:dir:attribute-def:eduPersonPrimaryAffiliation',
        'eduPersonPrimaryOrgUnitDN': 'urn:mace:dir:attribute-def:eduPersonPrimaryOrgUnitDN',
        'eduPersonPrincipalName': 'urn:mace:dir:attribute-def:eduPersonPrincipalName',
        'eduPersonScopedAffiliation': 'urn:mace:dir:attribute-def:eduPersonScopedAffiliation',
        'eduPersonTargetedID': 'urn:mace:dir:attribute-def:eduPersonTargetedID',
        'email': 'urn:mace:dir:attribute-def:email',
        'emailAddress': 'urn:mace:dir:attribute-def:emailAddress',
        'employeeNumber': 'urn:mace:dir:attribute-def:employeeNumber',
        'employeeType': 'urn:mace:dir:attribute-def:employeeType',
        'enhancedSearchGuide': 'urn:mace:dir:attribute-def:enhancedSearchGuide',
        'facsimileTelephoneNumber': 'urn:mace:dir:attribute-def:facsimileTelephoneNumber',
        'favouriteDrink': 'urn:mace:dir:attribute-def:favouriteDrink',
        'fax': 'urn:mace:dir:attribute-def:fax',
        'federationFeideSchemaVersion': 'urn:mace:dir:attribute-def:federationFeideSchemaVersion',
        'friendlyCountryName': 'urn:mace:dir:attribute-def:friendlyCountryName',
        'generationQualifier': 'urn:mace:dir:attribute-def:generationQualifier',
        'givenName': 'urn:mace:dir:attribute-def:givenName',
        'gn': 'urn:mace:dir:attribute-def:gn',
        'homePhone': 'urn:mace:dir:attribute-def:homePhone',
        'homePostalAddress': 'urn:mace:dir:attribute-def:homePostalAddress',
        'homeTelephoneNumber': 'urn:mace:dir:attribute-def:homeTelephoneNumber',
        'host': 'urn:mace:dir:attribute-def:host',
        'houseIdentifier': 'urn:mace:dir:attribute-def:houseIdentifier',
        'info': 'urn:mace:dir:attribute-def:info',
        'initials': 'urn:mace:dir:attribute-def:initials',
        'internationaliSDNNumber': 'urn:mace:dir:attribute-def:internationaliSDNNumber',
        'janetMailbox': 'urn:mace:dir:attribute-def:janetMailbox',
        'jpegPhoto': 'urn:mace:dir:attribute-def:jpegPhoto',
        'knowledgeInformation': 'urn:mace:dir:attribute-def:knowledgeInformation',
        'l': 'urn:mace:dir:attribute-def:l',
        'labeledURI': 'urn:mace:dir:attribute-def:labeledURI',
        'localityName': 'urn:mace:dir:attribute-def:localityName',
        'mDRecord': 'urn:mace:dir:attribute-def:mDRecord',
        'mXRecord': 'urn:mace:dir:attribute-def:mXRecord',
        'mail': 'urn:mace:dir:attribute-def:mail',
        'mailPreferenceOption': 'urn:mace:dir:attribute-def:mailPreferenceOption',
        'manager': 'urn:mace:dir:attribute-def:manager',
        'member': 'urn:mace:dir:attribute-def:member',
        'mobile': 'urn:mace:dir:attribute-def:mobile',
        'mobileTelephoneNumber': 'urn:mace:dir:attribute-def:mobileTelephoneNumber',
        'nSRecord': 'urn:mace:dir:attribute-def:nSRecord',
        'name': 'urn:mace:dir:attribute-def:name',
        'norEduOrgAcronym': 'urn:mace:dir:attribute-def:norEduOrgAcronym',
        'norEduOrgNIN': 'urn:mace:dir:attribute-def:norEduOrgNIN',
        'norEduOrgSchemaVersion': 'urn:mace:dir:attribute-def:norEduOrgSchemaVersion',
        'norEduOrgUniqueIdentifier': 'urn:mace:dir:attribute-def:norEduOrgUniqueIdentifier',
        'norEduOrgUniqueNumber': 'urn:mace:dir:attribute-def:norEduOrgUniqueNumber',
        'norEduOrgUnitUniqueIdentifier': 'urn:mace:dir:attribute-def:norEduOrgUnitUniqueIdentifier',
        'norEduOrgUnitUniqueNumber': 'urn:mace:dir:attribute-def:norEduOrgUnitUniqueNumber',
        'norEduPersonBirthDate': 'urn:mace:dir:attribute-def:norEduPersonBirthDate',
        'norEduPersonLIN': 'urn:mace:dir:attribute-def:norEduPersonLIN',
        'norEduPersonNIN': 'urn:mace:dir:attribute-def:norEduPersonNIN',
        'o': 'urn:mace:dir:attribute-def:o',
        'objectClass': 'urn:mace:dir:attribute-def:objectClass',
        'organizationName': 'urn:mace:dir:attribute-def:organizationName',
        'organizationalStatus': 'urn:mace:dir:attribute-def:organizationalStatus',
        'organizationalUnitName': 'urn:mace:dir:attribute-def:organizationalUnitName',
        'otherMailbox': 'urn:mace:dir:attribute-def:otherMailbox',
        'ou': 'urn:mace:dir:attribute-def:ou',
        'owner': 'urn:mace:dir:attribute-def:owner',
        'pager': 'urn:mace:dir:attribute-def:pager',
        'pagerTelephoneNumber': 'urn:mace:dir:attribute-def:pagerTelephoneNumber',
        'personalSignature': 'urn:mace:dir:attribute-def:personalSignature',
        'personalTitle': 'urn:mace:dir:attribute-def:personalTitle',
        'photo': 'urn:mace:dir:attribute-def:photo',
        'physicalDeliveryOfficeName': 'urn:mace:dir:attribute-def:physicalDeliveryOfficeName',
        'pkcs9email': 'urn:mace:dir:attribute-def:pkcs9email',
        'postOfficeBox': 'urn:mace:dir:attribute-def:postOfficeBox',
        'postalAddress': 'urn:mace:dir:attribute-def:postalAddress',
        'postalCode': 'urn:mace:dir:attribute-def:postalCode',
        'preferredDeliveryMethod': 'urn:mace:dir:attribute-def:preferredDeliveryMethod',
        'preferredLanguage': 'urn:mace:dir:attribute-def:preferredLanguage',
        'presentationAddress': 'urn:mace:dir:attribute-def:presentationAddress',
        'protocolInformation': 'urn:mace:dir:attribute-def:protocolInformation',
        'pseudonym': 'urn:mace:dir:attribute-def:pseudonym',
        'registeredAddress': 'urn:mace:dir:attribute-def:registeredAddress',
        'rfc822Mailbox': 'urn:mace:dir:attribute-def:rfc822Mailbox',
        'roleOccupant': 'urn:mace:dir:attribute-def:roleOccupant',
        'roomNumber': 'urn:mace:dir:attribute-def:roomNumber',
        'sOARecord': 'urn:mace:dir:attribute-def:sOARecord',
        'searchGuide': 'urn:mace:dir:attribute-def:searchGuide',
        'secretary': 'urn:mace:dir:attribute-def:secretary',
        'seeAlso': 'urn:mace:dir:attribute-def:seeAlso',
        'serialNumber': 'urn:mace:dir:attribute-def:serialNumber',
        'singleLevelQuality': 'urn:mace:dir:attribute-def:singleLevelQuality',
        'sn': 'urn:mace:dir:attribute-def:sn',
        'st': 'urn:mace:dir:attribute-def:st',
        'stateOrProvinceName': 'urn:mace:dir:attribute-def:stateOrProvinceName',
        'street': 'urn:mace:dir:attribute-def:street',
        'streetAddress': 'urn:mace:dir:attribute-def:streetAddress',
        'subtreeMaximumQuality': 'urn:mace:dir:attribute-def:subtreeMaximumQuality',
        'subtreeMinimumQuality': 'urn:mace:dir:attribute-def:subtreeMinimumQuality',
        'supportedAlgorithms': 'urn:mace:dir:attribute-def:supportedAlgorithms',
        'supportedApplicationContext': 'urn:mace:dir:attribute-def:supportedApplicationContext',
        'surname': 'urn:mace:dir:attribute-def:surname',
        'telephoneNumber': 'urn:mace:dir:attribute-def:telephoneNumber',
        'teletexTerminalIdentifier': 'urn:mace:dir:attribute-def:teletexTerminalIdentifier',
        'telexNumber': 'urn:mace:dir:attribute-def:telexNumber',
        'textEncodedORAddress': 'urn:mace:dir:attribute-def:textEncodedORAddress',
        'title': 'urn:mace:dir:attribute-def:title',
        'uid': 'urn:mace:dir:attribute-def:uid',
        'uniqueIdentifier': 'urn:mace:dir:attribute-def:uniqueIdentifier',
        'uniqueMember': 'urn:mace:dir:attribute-def:uniqueMember',
        'userCertificate': 'urn:mace:dir:attribute-def:userCertificate',
        'userClass': 'urn:mace:dir:attribute-def:userClass',
        'userPKCS12': 'urn:mace:dir:attribute-def:userPKCS12',
        'userPassword': 'urn:mace:dir:attribute-def:userPassword',
        'userSMIMECertificate': 'urn:mace:dir:attribute-def:userSMIMECertificate',
        'userid': 'urn:mace:dir:attribute-def:userid',
        'x121Address': 'urn:mace:dir:attribute-def:x121Address',
        'x500UniqueIdentifier': 'urn:mace:dir:attribute-def:x500UniqueIdentifier',
    }
}